/*
    This file is part of AirSnort.

    AirSnort is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    AirSnort is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AirSnort; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>
#include "RC4.h"
#include "pcap.h"

//data that varies based on what type of packet it is
typedef struct CaptureRec_t {
  unsigned char *raw;     //pointer to common data
  unsigned char *bssid;  //pointer to the AP bssid
  unsigned char *sta;    //pointer to the sta 
  unsigned char *iv;     //pointer to the frame body
} CaptureRec;


int eatHexString(char *dst,char *src,int maxlen);
void hexdump(unsigned char *buf,int size);

#define ADDR1(x) (x) + offset + 4
#define ADDR2(x) (x) + offset + 10
#define ADDR3(x) (x) + offset + 16

//initialize data in four capture records according to TO_DS,
//FROM_DS flags.  This saves time during packet decoding
void initCapRecs(CaptureRec *cr, unsigned char *packet, int offset) {
   int i;
   for (i = 0; i < 4; i++) {
      cr[i].raw = packet;
      cr[i].iv = packet + offset + 24;
      switch (i) {
         case 0:  //STA to STA, or management and control
            cr[0].bssid = ADDR3(packet);
            cr[0].sta =  ADDR2(packet);
            break;
         case 2:  //AP to STA data
            cr[2].bssid = ADDR2(packet);
            cr[2].sta = ADDR1(packet); 
            break;
         case 3:  //AP to AP 
            cr[3].iv += 6;
         case 1:  //STA to AP data
            cr[i].bssid = ADDR1(packet);
            cr[i].sta = ADDR2(packet); 
            break;
      }
   }
}

char hexdecode[]="0123456789ABCDEF";

int eatHexByte(char *x) {
  int r;
  if (x[0] >= '0' && x[0] <= '9') r = (x[0] - '0') * 16;
  else if (x[0] >= 'A' && x[0] <= 'F') r = (x[0] - 'A' + 10) * 16;
  else return (-1);
  if (x[1] >= '0' && x[1] <= '9') r += x[1] - '0';
  else if (x[1] >= 'A' && x[1] <= 'F') r += x[1] - 'A' + 10;
  else return (-1);
  return (r);
}

int eatHexString(char *dst, char *src, int maxlen) {
  int l = 0;
  int r;
  while (l < maxlen) {
    r = eatHexByte(src);
    if (r < 0) return(-1);
    dst[l++] = r;
    src += 2;
    if (*src == 0) return(l);
    if (*src == ':') src++;
    else return (-1);
  }
  return(-1);
}

int matchMac(const unsigned char *bssid1, const unsigned char *bssid2) {
   int i = 5;
   for (; i >= 0; i--) {
      if (bssid1[i] != bssid2[i]) return 0;
   }
   return 1;
}

void usage(char *cmd) {
    fprintf(stderr, 
        "Usage: %s (-s <pw> | -h <pw>) [-o <offset>] -m <bssid> -e <cryptfile> -d <decryptfile>\n", 
        cmd);
    exit(1);
}

int main(int argc,char *argv[]) {
   int r, i, infile, len, plen, outfile, offset = 0;
   unsigned char password[16];
   unsigned char packet[4000];
   char *cryptfile, *decryptfile;
   unsigned char bssid[6]; 
   RC4 rc;
   struct pcap_file_header hdr;
   struct pcap_pkthdr pkt;
   int havePW = 0, haveMac = 0, haveIn = 0, haveOut = 0;

   int whichType;
   CaptureRec cap[4], *thePacket;
   unsigned char fc0 = 0, type = 0, subType = 0;

   setupIdentity();
   plen = 13;
   memset(password, 0, 16);

   while (1) {
      r = getopt(argc,argv,"s:h:o:m:e:d:");
      if (r < 0) break;
      switch (r) {
      case 's':
         if (strlen(optarg) == 13) {
            memcpy(password+3, optarg, 13);
         } else if (strlen(optarg) == 5) {
	    memcpy(password + 3, optarg, 5);
	    plen = 5;
         } else {
	    fprintf(stderr,
		"Invalid password length: must be 5 or 13 characters\n");
            exit(1);
         }
         havePW = 1;
         break;
      case 'h':
         r = eatHexString((char *) password + 3, optarg, 13);
         if (r != 13 && r != 5) {
	   fprintf(stderr, "Invalid password\n");
         }
         plen = 5;
         havePW = 1;
         break;
      case 'o':
         offset = atoi(optarg);
         break;
      case 'm':
         if (eatHexString((char *) bssid, optarg, 6) != 6) {
	    fprintf(stderr, "Invalid bssid, quitting\n");
            exit(1);
         }
         haveMac = 1;
         break;
      case 'e':
         cryptfile = optarg;
         haveIn = 1;
         break;
      case 'd':
         decryptfile = optarg;
         haveOut = 1;
         break;
      default:
         usage(argv[0]);
      }
   } 
   if (!(haveIn && haveOut && haveMac && havePW)) {
      fprintf(stderr, "missing required argument\n");
      usage(argv[0]);
   }

   infile = open(cryptfile, O_RDONLY);
   if (infile < 0) {
      perror("Unable to open cryptfile");
      exit(1);
   }
   outfile = open(decryptfile, O_WRONLY | O_CREAT, 0644);
   if (outfile < 0) {
      perror("Unable to open decryptfile");
      exit(1);
   }
   readPcapHeader(infile, &hdr);
   write(outfile, &hdr, sizeof(hdr));
   switch (hdr.linktype) {
      case LINKTYPE_IEEE802_11:
         offset = 0;
         break;
      case LINKTYPE_PRISM_HEADER:
         offset = 144;
         break;
   }

   initCapRecs(cap, packet, offset);

   while ((len = readPcapPacket(infile, &pkt, packet, 4000)) > 0) {;
      whichType = packet[offset + 1] & 0x03;
      thePacket = &(cap[whichType]);
      fc0 = packet[offset];
      type = fc0 & 0x0C;
      subType = (fc0 >> 4);
      
      if (type == 0x08) {
         if (subType == 0) {
            if (matchMac(bssid, thePacket->bssid)) {
               len -= whichType == 3 ? (offset + 34) : (offset + 28);  //adjust len to size of frame body

               //this is a sloppy way to detect unencrypted packets, but since 0xAA is not
               //an IV byte of interest anyway, its probably not a big deal
               if (*((unsigned short *)thePacket->iv) != 0xAAAA || (packet[offset + 1] & 0x40)) {

                  memcpy(password, thePacket->iv, 3);
                  RC4init(&rc);
                  keyWith(&rc, password, 3 + plen);
                  for (i = 4; i < len; i++) {
                     thePacket->iv[i - 4] = thePacket->iv[i] ^ step(&rc);
                  }
                  pkt.len -= 8;
                  pkt.caplen -= 8;
               }
            }
         }
      } //if
      writePcapPacket(outfile, &pkt, packet);
   }

   close(infile);
   close(outfile);
}

